<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Cashier;
use App\Models\FinancialCategory;
use App\Models\GeneralLedger;
use App\Models\Header;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;
use Gate;

class GeneralLedgerController extends Controller
{
    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->branch_id = session()->get('branch_id');
            return $next($request);
        });

    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('general_ledger_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $query = GeneralLedger::query()->where('branch_id',$this->branch_id)->with('subcategories','cashier')->get();

        if ($request->ajax()) {
            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');
            $table->editColumn('date',function($row){
                return $row->date ? date('d/m/Y',strtotime($row->date)) :'';
            });
            $table->editColumn('sub_category',function($row){
                return $row->subcategories ? $row->subcategories->title : '';
            });
            $table->editColumn('cashier',function($row){
                return $row->cashier ? $row->cashier->name : '';
            });
            $table->editColumn('actions', function ($row) {
                $viewGate = 'general_ledger_show';
                $editGate = 'general_ledger_edit';
                $deleteGate = 'general_ledger_delete';

                $crudRoutePart = 'general-ledgers';

                return view('partials.datatablesActions', compact(
                    'viewGate',
                    'editGate',
                    'deleteGate',
                    'crudRoutePart',
                    'row'
                ));
            });
            $table->rawColumns(['actions','placeholder']);
            return $table->make(true);
        }
        return view('admin.generalLedger.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('general_ledger_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $category = Header::query()->with('subcategories')->where('category_type','other')->get();
        $sub_category = new Collection();
        foreach ($category as $cat) {
            $sub_category = $sub_category->merge($cat->subcategories->where('status',1));
        }
        $sub_category = $sub_category->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        $cashier = collect();

        return view('admin.generalLedger.create',compact('sub_category','cashier'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        abort_if(Gate::denies('general_ledger_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $request->validate([
            'date' => 'required',
            'sub_category.*' => 'required|integer',
            'amount.*' => 'required|numeric',
            'cashier.*' => 'nullable|integer',
            'detail.*' => 'nullable',
        ]);

        $date = date('Y-m-d',strtotime(strtr($request->date,'/','-')));

        foreach ($request->amount as $key => $value) {
            $cashier = null;
            if ($request->has('cashier') && $request->cashier != null) {
                $cashier = $request->cashier[$key];
            }
            GeneralLedger::create([
                'date' => $date,
                'sub_category_id' => $request->sub_category[$key],
                'cashier_id' => $cashier,
                'amount' => $request->amount[$key],
                'branch_id' => $this->branch_id,
                'detail' => $request->detail[$key],
            ]);
        }

        if ($request->ajax()) {
            return response()->json(['success' => true,'message' => 'General ledger is created successfully!','url'=> route('admin.general-ledgers.index')]);
        }
        else{
            toastr()->success('General ledger is created successfully!');
            return redirect()->route('admin.general-ledgers.index');
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(GeneralLedger $generalLedger)
    {
        abort_if(Gate::denies('general_ledger_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.generalLedger.show',compact('generalLedger'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(GeneralLedger $generalLedger)
    {
        abort_if(Gate::denies('general_ledger_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $category = Header::query()->with('subcategories')->where('category_type','other')->get();
        $sub_category = new Collection();
        foreach ($category as $cat) {
            $sub_category = $sub_category->merge($cat->subcategories->where('status',1));
        }
        $sub_category = $sub_category->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        $cashier = Cashier::where('branch_id',$this->branch_id)->where('status',1)->get()->pluck('name','id')->prepend(trans('global.pleaseSelect'),'');
        return view('admin.generalLedger.edit',compact('generalLedger','sub_category','cashier'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, GeneralLedger $generalLedger)
    {
        abort_if(Gate::denies('general_ledger_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $request->validate([
            'date' => 'required',
            'sub_category' => 'required|integer',
            'amount' => 'required|numeric',
            'cashier' => 'nullable|integer',
            'detail' => 'nullable',
        ]);

        $date = date('Y-m-d',strtotime(strtr($request->date,'/','-')));

        $generalLedger->update([
            'date' => $date,
            'sub_category_id' => $request->sub_category,
            'cashier_id' => $request->cashier,
            'amount' => $request->amount,
            'branch_id' => $this->branch_id,
            'detail' => $request->detail,
        ]);
        toastr()->info('General ledger has been updated successfully!');
        return redirect()->route('admin.general-ledgers.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(GeneralLedger $generalLedger)
    {
        abort_if(Gate::denies('general_ledger_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $generalLedger->forceDelete();
        toastr()->error('General Ledger has been deleted!');
        return back();
    }

    public function fetch_cashier($id)
    {
        $cat = FinancialCategory::where('id',$id)->first();
        $cashiers = Cashier::where('branch_id',$this->branch_id)->where('status',1)->where('type','cashier')->get()->pluck('name','id');
        if($cat->code == 'sub47'){
            $cashiers = Cashier::where('branch_id',$this->branch_id)->where('status',1)->where('type','attendent')->get()->pluck('name','id');
        }
        return json_encode($cashiers);
    }

    public function massDestroy(Request $request)
    {
        abort_if(Gate::denies('general_ledger_delete'),Response::HTTP_FORBIDDEN,'403 Forbidden');
        $request->validate([
            'ids'   => 'required|array',
            'ids.*' => 'exists:general_ledgers,id',
        ]);

        foreach($request->ids as $id){
            $generalLedger = GeneralLedger::find($id);
            $generalLedger->forceDelete();
        }
        toastr()->error('General ledger has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
}
